@@ -31,6 +31,8 @@
 /**
  * @struct sqfs_dir_writer_t
  *
+ * @implements sqfs_object_t
+ *
  * @brief Abstracts generating of directory entries
  *
  * SquashFS stores directory entries and inodes separated from each other. The
@@ -56,6 +58,8 @@
  * adding entries. Internally it fills data into a meta data writer and
  * generates an index that it can, on request, write to another meta data
  * writer used for inodes.
+ *
+ * This object is not copyable, i.e. @ref sqfs_copy will always return NULL.
  */
 
 /**
@@ -101,15 +105,6 @@ extern "C" {
 SQFS_API sqfs_dir_writer_t *sqfs_dir_writer_create(sqfs_meta_writer_t *dm,
 						   sqfs_u32 flags);
 
-/**
- * @brief Destroy a directory writer and free all its memory.
- *
- * @memberof sqfs_dir_writer_t
- *
- * @param writer A pointer to a directory writer object.
- */
-SQFS_API void sqfs_dir_writer_destroy(sqfs_dir_writer_t *writer);
-
 /**
  * @brief Begin writing a directory, i.e. reset and initialize all internal
  *        state neccessary.
@@ -119,7 +114,7 @@ SQFS_API void sqfs_dir_writer_destroy(sqfs_dir_writer_t *writer);
  * @param writer A pointer to a directory writer object.
  * @param flags A currently unused flag field. Must be set to 0.
  *
- * @return Zero on success, a @ref E_SQFS_ERROR value on failure.
+ * @return Zero on success, a @ref SQFS_ERROR value on failure.
  */
 SQFS_API int sqfs_dir_writer_begin(sqfs_dir_writer_t *writer, sqfs_u32 flags);
 
@@ -137,7 +132,7 @@ SQFS_API int sqfs_dir_writer_begin(sqfs_dir_writer_t *writer, sqfs_u32 flags);
  * @param mode A file mode, i.e. type and permission bits from which the entry
  *             type is derived internally.
  *
- * @return Zero on success, a @ref E_SQFS_ERROR value on failure.
+ * @return Zero on success, a @ref SQFS_ERROR value on failure.
  */
 SQFS_API int sqfs_dir_writer_add_entry(sqfs_dir_writer_t *writer,
 				       const char *name,
@@ -152,7 +147,7 @@ SQFS_API int sqfs_dir_writer_add_entry(sqfs_dir_writer_t *writer,
  *
  * @param writer A pointer to a directory writer object.
  *
- * @return Zero on success, a @ref E_SQFS_ERROR value on failure.
+ * @return Zero on success, a @ref SQFS_ERROR value on failure.
  */
 SQFS_API int sqfs_dir_writer_end(sqfs_dir_writer_t *writer);
 
@@ -270,7 +265,7 @@ SQFS_API sqfs_inode_generic_t
  * @param super A pointer to the super block. Location of the export table and
  *              the exportable flag are set.
  *
- * @return Zero on success, a @ref E_SQFS_ERROR value on failure.
+ * @return Zero on success, a @ref SQFS_ERROR value on failure.
  */
 SQFS_API int sqfs_dir_writer_write_export_table(sqfs_dir_writer_t *writer,
 						sqfs_file_t *file,
