@@ -29,7 +29,7 @@
  */
 
 /**
- * @enum E_SQFS_INODE_TYPE
+ * @enum SQFS_INODE_TYPE
  *
  * @brief Used by @ref sqfs_inode_t to identify the inode type.
  */
@@ -48,10 +48,10 @@ typedef enum {
 	SQFS_INODE_EXT_CDEV = 12,
 	SQFS_INODE_EXT_FIFO = 13,
 	SQFS_INODE_EXT_SOCKET = 14,
-} E_SQFS_INODE_TYPE;
+} SQFS_INODE_TYPE;
 
 /**
- * @enum E_SQFS_INODE_MODE
+ * @enum SQFS_INODE_MODE
  *
  * @brief Mode bits for the @ref sqfs_inode_t mode field.
  *
@@ -120,7 +120,7 @@ typedef enum {
 	SQFS_INODE_MODE_LNK = 0120000,
 	SQFS_INODE_MODE_SOCK = 0140000,
 	SQFS_INODE_MODE_MASK = 0170000,
-} E_SQFS_INODE_MODE;
+} SQFS_INODE_MODE;
 
 /**
  * @struct sqfs_inode_t
@@ -132,7 +132,7 @@ typedef enum {
  */
 struct sqfs_inode_t {
 	/**
-	 * @brief An @ref E_SQFS_INODE_TYPE value.
+	 * @brief An @ref SQFS_INODE_TYPE value.
 	 */
 	sqfs_u16 type;
 
@@ -140,7 +140,7 @@ struct sqfs_inode_t {
 	 * @brief Mode filed holding permission bits only. The type is derived
 	 *        from the type field.
 	 *
-	 * This field holds a combination of @ref E_SQFS_INODE_MODE flags.
+	 * This field holds a combination of @ref SQFS_INODE_MODE flags.
 	 */
 	sqfs_u16 mode;
 
@@ -447,8 +447,8 @@ struct sqfs_inode_dir_ext_t {
 	/**
 	 * @brief Number of directory index entries following the inode
 	 *
-	 * This number is stored off-by one and counts the number of
-	 * @ref sqfs_dir_index_t entries following the inode.
+	 * This number counts the number of @ref sqfs_dir_index_t entries
+	 * following the inode.
 	 */
 	sqfs_u16 inodex_count;
 
@@ -483,32 +483,21 @@ struct sqfs_inode_generic_t {
 	sqfs_inode_t base;
 
 	/**
-	 * @brief A pointer into the extra field holding the symlink target.
+	 * @brief Maximum number of available data bytes in the payload.
 	 *
-	 * @param This string is not null terminated. The helper functions rely
-	 *        entirely on the length stored in the symlink inode.
+	 * This is used for dynamically growing an inode. The actual number
+	 * of used payload bytes is stored in @ref payload_bytes_used.
 	 */
-	char *slink_target;
+	sqfs_u32 payload_bytes_available;
 
 	/**
-	 * @brief A pointer into the extra field holding file blocks sizes.
+	 * @brief Number of used data bytes in the payload.
 	 *
-	 * For file inodes, holds the consecutive block sizes. Bit number 24 is
-	 * set if the block is stored uncompressed. If it the size is zero,
-	 * the block is sparse.
+	 * For file inodes, stores the number of blocks used. For extended
+	 * directory inodes, stores the number of payload bytes following
+	 * for the directory index.
 	 */
-	sqfs_u32 *block_sizes;
-
-	/**
-	 * @brief For file inodes, stores the number of blocks used.
-	 */
-	size_t num_file_blocks;
-
-	/**
-	 * @brief For extended directory inodes, stores the number of payload
-	 *        bytes following for the directory index.
-	 */
-	size_t num_dir_idx_bytes;
+	sqfs_u32 payload_bytes_used;
 
 	/**
 	 * @brief Type specific inode data.
@@ -528,17 +517,39 @@ struct sqfs_inode_generic_t {
 
 	/**
 	 * @brief Holds type specific extra data, such as symlink target.
+	 *
+	 * For regular file inodes, this is an array of block sizes. For symlink
+	 * inodes, this is actually a string holding the target. For extended
+	 * directory inodes, this is actually a blob of tightly packed directory
+	 * index entries.
 	 */
-	sqfs_u8 extra[];
+	sqfs_u32 extra[];
 };
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/**
+ * @brief Get the number of file blocks in a regular file inode.
+ *
+ * @memberof sqfs_inode_generic_t
+ *
+ * @param inode A pointer to an inode.
+ *
+ * @return The number of blocks.
+ */
+static SQFS_INLINE
+size_t sqfs_inode_get_file_block_count(const sqfs_inode_generic_t *inode)
+{
+	return inode->payload_bytes_used / sizeof(sqfs_u32);
+}
+
 /**
  * @brief Get the extended attribute index of an inode
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * For basic inodes, this returns the inode index 0xFFFFFFFF, i.e. the
  * sentinel value indicating that there are no xattrs.
  *
@@ -554,6 +565,8 @@ SQFS_API int sqfs_inode_get_xattr_index(const sqfs_inode_generic_t *inode,
 /**
  * @brief Set the extended attribute index of an inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * For basic inodes, this function promes the inodes to extended inodes if the
  * index is not 0xFFFFFFFF. If the index is 0xFFFFFFFF, the function tries to
  * demote extended inode to a basic inode after setting the index.
@@ -570,6 +583,8 @@ SQFS_API int sqfs_inode_set_xattr_index(sqfs_inode_generic_t *inode,
 /**
  * @brief Convert a basic inode to an extended inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * For inodes that already have an extended type, this is a no-op.
  *
  * @param inode A pointer to an inode.
@@ -582,6 +597,8 @@ SQFS_API int sqfs_inode_make_extended(sqfs_inode_generic_t *inode);
 /**
  * @brief Convert an extended inode to a basic inode if possible.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * For inodes that already have a basic type, this is a no-op. If the inode
  * has values set that the coresponding basic type doesn't support (e.g. it
  * has an xattr index set or a regular file which requires 64 bit size
@@ -597,6 +614,8 @@ SQFS_API int sqfs_inode_make_basic(sqfs_inode_generic_t *inode);
 /**
  * @brief Update the file size of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * If the new size is wider than 32 bit, a basic file inode is transparently
  * promoted to an extended file inode. For extended inodes, if the new size
  * is small enough and was the only requirement for the extended type, the
@@ -614,6 +633,8 @@ SQFS_API int sqfs_inode_set_file_size(sqfs_inode_generic_t *inode,
 /**
  * @brief Update the location of the first data block of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * If the new location is wider than 32 bit, a basic file inode is
  * transparently promoted to an extended file inode. For extended inodes,
  * if the new size is small enough and was the only requirement for the
@@ -631,6 +652,8 @@ SQFS_API int sqfs_inode_set_file_block_start(sqfs_inode_generic_t *inode,
 /**
  * @brief Update the file fragment location of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * @param inode A pointer to an inode.
  * @param index The new fragment index to set.
  * @param offset The new fragment offset to set.
@@ -644,6 +667,8 @@ SQFS_API int sqfs_inode_set_frag_location(sqfs_inode_generic_t *inode,
 /**
  * @brief Get the file size of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * @param inode A pointer to an inode.
  * @param size Returns the file size.
  *
@@ -656,6 +681,8 @@ SQFS_API int sqfs_inode_get_file_size(const sqfs_inode_generic_t *inode,
 /**
  * @brief Get the file fragment location of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * @param inode A pointer to an inode.
  * @param index Returns the fragment index.
  * @param offset Returns the fragment offset.
@@ -669,6 +696,8 @@ SQFS_API int sqfs_inode_get_frag_location(const sqfs_inode_generic_t *inode,
 /**
  * @brief Get the location of the first data block of a regular file inode.
  *
+ * @memberof sqfs_inode_generic_t
+ *
  * @param inode A pointer to an inode.
  * @param location Returns the location.
  *
@@ -678,6 +707,29 @@ SQFS_API int sqfs_inode_get_frag_location(const sqfs_inode_generic_t *inode,
 SQFS_API int sqfs_inode_get_file_block_start(const sqfs_inode_generic_t *inode,
 					     sqfs_u64 *location);
 
+/**
+ * @brief Unpack the a directory index structure from an inode.
+ *
+ * @memberof sqfs_inode_generic_t
+ *
+ * The generic inode contains in its payload the raw directory index (with
+ * bytes swapped to host enian), but still with single byte alignment. This
+ * function seeks through the blob using an integer index (not offset) and
+ * fiddles the raw data out into a propperly aligned, external structure.
+ *
+ * @param inode A pointer to an inode.
+ * @param out Returns the index entry. Can be freed with a single free call.
+ * @param index An index value between 0 and inodex_count.
+ *
+ * @return Zero on success, @ref SQFS_ERROR_OUT_OF_BOUNDS if the given index
+ *         points outside the directory index. Can return other error codes
+ *         (e.g. if the inode isn't a directory or allocation failed).
+ */
+SQFS_API
+int sqfs_inode_unpack_dir_index_entry(const sqfs_inode_generic_t *inode,
+				      sqfs_dir_index_t **out,
+				      size_t index);
+
 #ifdef __cplusplus
 }
 #endif
