@@ -5,9 +5,7 @@
  * Copyright (C) 2019 David Oberhollenzer <goliath@infraroot.at>
  * Copyright (C) 2019 Zachary Dremann <dremann@gmail.com>
  */
-#include "config.h"
-
-#include "fstree.h"
+#include "internal.h"
 
 #include <string.h>
 
@@ -60,13 +58,3 @@ tree_node_t *tree_node_list_sort(tree_node_t *head)
 
 	return merge(tree_node_list_sort(head), tree_node_list_sort(half));
 }
-
-void tree_node_sort_recursive(tree_node_t *n)
-{
-	n->data.dir.children = tree_node_list_sort(n->data.dir.children);
-
-	for (n = n->data.dir.children; n != NULL; n = n->next) {
-		if (S_ISDIR(n->mode))
-			tree_node_sort_recursive(n);
-	}
-}
