@@ -37,6 +37,8 @@ typedef struct index_ent_t {
 } index_ent_t;
 
 struct sqfs_dir_writer_t {
+	sqfs_object_t base;
+
 	dir_entry_t *list;
 	dir_entry_t *list_end;
 
@@ -136,6 +138,15 @@ static int add_export_table_entry(sqfs_dir_writer_t *writer,
 	return 0;
 }
 
+static void dir_writer_destroy(sqfs_object_t *obj)
+{
+	sqfs_dir_writer_t *writer = (sqfs_dir_writer_t *)obj;
+
+	writer_reset(writer);
+	free(writer->export_tbl);
+	free(writer);
+}
+
 sqfs_dir_writer_t *sqfs_dir_writer_create(sqfs_meta_writer_t *dm,
 					  sqfs_u32 flags)
 {
@@ -162,17 +173,11 @@ sqfs_dir_writer_t *sqfs_dir_writer_create(sqfs_meta_writer_t *dm,
 		       sizeof(writer->export_tbl[0]) * writer->export_tbl_max);
 	}
 
+	((sqfs_object_t *)writer)->destroy = dir_writer_destroy;
 	writer->dm = dm;
 	return writer;
 }
 
-void sqfs_dir_writer_destroy(sqfs_dir_writer_t *writer)
-{
-	writer_reset(writer);
-	free(writer->export_tbl);
-	free(writer);
-}
-
 int sqfs_dir_writer_begin(sqfs_dir_writer_t *writer, sqfs_u32 flags)
 {
 	sqfs_u32 offset;
@@ -387,6 +392,7 @@ sqfs_inode_generic_t
 	if (inode == NULL)
 		return NULL;
 
+	inode->payload_bytes_available = index_size;
 	start_block = writer->dir_ref >> 16;
 	block_offset = writer->dir_ref & 0xFFFF;
 
@@ -411,7 +417,7 @@ sqfs_inode_generic_t
 		inode->data.dir_ext.offset = block_offset;
 		inode->data.dir_ext.xattr_idx = xattr;
 		inode->data.dir_ext.inodex_count = 0;
-		inode->num_dir_idx_bytes = 0;
+		inode->payload_bytes_used = 0;
 
 		for (idx = writer->idx; idx != NULL; idx = idx->next) {
 			memset(&ent, 0, sizeof(ent));
@@ -419,14 +425,15 @@ sqfs_inode_generic_t
 			ent.index = idx->index;
 			ent.size = idx->ent->name_len - 1;
 
-			ptr = inode->extra + inode->num_dir_idx_bytes;
+			ptr = (sqfs_u8 *)inode->extra +
+				inode->payload_bytes_used;
 			memcpy(ptr, &ent, sizeof(ent));
 			memcpy(ptr + sizeof(ent), idx->ent->name,
 			       idx->ent->name_len);
 
 			inode->data.dir_ext.inodex_count += 1;
-			inode->num_dir_idx_bytes += sizeof(ent);
-			inode->num_dir_idx_bytes += idx->ent->name_len;
+			inode->payload_bytes_used += sizeof(ent);
+			inode->payload_bytes_used += idx->ent->name_len;
 		}
 	}
 
