@@ -17,22 +17,62 @@
 #include <string.h>
 
 struct sqfs_id_table_t {
+	sqfs_object_t base;
+
 	sqfs_u32 *ids;
 	size_t num_ids;
 	size_t max_ids;
 };
 
-sqfs_id_table_t *sqfs_id_table_create(void)
+static void id_table_destroy(sqfs_object_t *obj)
 {
-	return calloc(1, sizeof(sqfs_id_table_t));
-}
+	sqfs_id_table_t *tbl = (sqfs_id_table_t *)obj;
 
-void sqfs_id_table_destroy(sqfs_id_table_t *tbl)
-{
 	free(tbl->ids);
 	free(tbl);
 }
 
+static sqfs_object_t *id_table_copy(const sqfs_object_t *obj)
+{
+	const sqfs_id_table_t *tbl = (const sqfs_id_table_t *)obj;
+	sqfs_id_table_t *copy;
+
+	copy = malloc(sizeof(*copy));
+	if (copy == NULL)
+		return NULL;
+
+	memcpy(copy, tbl, sizeof(*tbl));
+
+	copy->num_ids = tbl->num_ids;
+	copy->max_ids = tbl->num_ids;
+	copy->ids = malloc(tbl->num_ids * sizeof(tbl->ids[0]));
+
+	if (copy->ids == NULL) {
+		free(copy);
+		return NULL;
+	}
+
+	memcpy(copy->ids, tbl->ids, tbl->num_ids * sizeof(tbl->ids[0]));
+	return (sqfs_object_t *)copy;
+}
+
+sqfs_id_table_t *sqfs_id_table_create(sqfs_u32 flags)
+{
+	sqfs_id_table_t *tbl;
+
+	if (flags != 0)
+		return NULL;
+
+	tbl = calloc(1, sizeof(sqfs_id_table_t));
+
+	if (tbl != NULL) {
+		((sqfs_object_t *)tbl)->destroy = id_table_destroy;
+		((sqfs_object_t *)tbl)->copy = id_table_copy;
+	}
+
+	return tbl;
+}
+
 int sqfs_id_table_id_to_index(sqfs_id_table_t *tbl, sqfs_u32 id, sqfs_u16 *out)
 {
 	size_t i, sz;
