@@ -27,6 +27,8 @@ enum {
 	PAX_SLINK_TARGET = 0x040,
 	PAX_MTIME = 0x100,
 	PAX_SPARSE_SIZE = 0x400,
+
+	PAX_SPARSE_GNU_1_X = 0x800,
 };
 
 enum {
@@ -40,6 +42,7 @@ enum {
 #define TAR_MAX_SYMLINK_LEN (65536)
 #define TAR_MAX_PATH_LEN (65536)
 #define TAR_MAX_PAX_LEN (65536)
+#define TAR_MAX_SPARSE_ENT (65536)
 
 
 int read_octal(const char *str, int digits, sqfs_u64 *out);
@@ -58,6 +61,8 @@ sparse_map_t *read_sparse_map(const char *line);
 
 sparse_map_t *read_gnu_old_sparse(FILE *fp, tar_header_t *hdr);
 
+sparse_map_t *read_gnu_new_sparse(FILE *fp, tar_header_decoded_t *out);
+
 void free_sparse_list(sparse_map_t *sparse);
 
 size_t base64_decode(sqfs_u8 *out, const char *in, size_t len);
