@@ -116,7 +116,8 @@ static int decode_header(const tar_header_t *hdr, unsigned int set_by_pax,
 	if (hdr->typeflag == TAR_TYPE_LINK ||
 	    hdr->typeflag == TAR_TYPE_SLINK) {
 		if (!(set_by_pax & PAX_SLINK_TARGET)) {
-			out->link_target = strdup(hdr->linkname);
+			out->link_target = strndup(hdr->linkname,
+						   sizeof(hdr->linkname));
 			if (out->link_target == NULL) {
 				perror("decoding symlink target");
 				return -1;
@@ -254,6 +255,13 @@ int read_header(FILE *fp, tar_header_decoded_t *out)
 	if (decode_header(&hdr, set_by_pax, out, version))
 		goto fail;
 
+	if (set_by_pax & PAX_SPARSE_GNU_1_X) {
+		free_sparse_list(out->sparse);
+		out->sparse = read_gnu_new_sparse(fp, out);
+		if (out->sparse == NULL)
+			goto fail;
+	}
+
 	if (out->sparse != NULL) {
 		out->sb.st_size = out->actual_size;
 	} else {
