@@ -6,12 +6,11 @@
  */
 #include "config.h"
 
-#include "util/util.h"
 #include "tar.h"
 
 #include <stdio.h>
 
-static int skip_bytes(int fd, sqfs_u64 size)
+static int skip_bytes(FILE *fp, sqfs_u64 size)
 {
 	unsigned char buffer[1024];
 	size_t diff;
@@ -21,7 +20,7 @@ static int skip_bytes(int fd, sqfs_u64 size)
 		if (diff > size)
 			diff = size;
 
-		if (read_retry("reading tar record padding", fd, buffer, diff))
+		if (read_retry("reading tar record padding", fp, buffer, diff))
 			return -1;
 
 		size -= diff;
@@ -30,16 +29,16 @@ static int skip_bytes(int fd, sqfs_u64 size)
 	return 0;
 }
 
-int skip_padding(int fd, sqfs_u64 size)
+int skip_padding(FILE *fp, sqfs_u64 size)
 {
 	size_t tail = size % 512;
 
-	return tail ? skip_bytes(fd, 512 - tail) : 0;
+	return tail ? skip_bytes(fp, 512 - tail) : 0;
 }
 
-int skip_entry(int fd, sqfs_u64 size)
+int skip_entry(FILE *fp, sqfs_u64 size)
 {
 	size_t tail = size % 512;
 
-	return skip_bytes(fd, tail ? (size + 512 - tail) : size);
+	return skip_bytes(fp, tail ? (size + 512 - tail) : size);
 }
